
#include <msp430.h>
#include <stdint.h>

//-- Hardver absztrakci -------------
#define LED1      BIT0
#define TXD       BIT1                 // TXD a P1.1 lbon
#define SW2       BIT3                 // S2 nyomgomb
#define CSEL      BIT4                 // SPI Chip Select
#define SCK       BIT5                 // SPI Clock
#define SDI       BIT6                 // SPI Data input

uint32_t data; 
uint16_t d1,d2;
uint8_t counter,
        adc_sign,                      // eljel bit trolja
        adc_ext;                       // kiterjesztett tartomny jelzje
int32_t u;
uint64_t w;

/**------------------------------------------------------------
 *   Egy karakter kikldse a soros portra
 *   SW_UART: 9600 bit/s, 8, N, 2 formtum
 *   DCO = 1 MHz (Bitid = 104.167 usec)
 *-------------------------------------------------------------
 * c - a kikldeni kvnt karakter kdja
 */
void sw_uart_putc(char c) {
  uint8_t i;
  uint16_t TXData;                //Adatregiszter (11 bites)
  TXData = (uint16_t)c | 0x300;   //2 Stop bit (mark)
  TXData = TXData << 1;           //Start bit (space)
  for(i=0; i<11; i++) {           //11 bitet kldnk ki
    if(TXData & 0x0001) {         //Soron kvetkez bit vizsglata
      P1OUT |= TXD;               //Ha '1'
    } else {
      P1OUT &= ~TXD;              //Ha '0'
    }
    TXData = TXData >> 1;         //Adatregiszter lptets jobbra
    __delay_cycles(89);           //<== Itt kell hangolni!
  }
  P1OUT |= TXD;                   //Az alaphelyzet: mark
}

/**------------------------------------------------------------
 *  Karakterfzr kirsa a soros portra
 *-------------------------------------------------------------
 * p_str - karakterfzr mutat (nullval lezrt stringre mutat)
 */
void sw_uart_puts(char* p_str) {
  char c;
  while ((c=*p_str)) {
      sw_uart_putc(c);
      p_str++;
  }
}

/**------------------------------------------------------------
 * Eljel nlkli egsz szm kirsa hexadecimlis formban
 *-------------------------------------------------------------
 * t - a kirand szm (uint16_t tpus)
 */
void sw_uart_out4hex(uint16_t t)
{
  char c;
  c=(char)((t>>12) & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
  c=(char)((t>>8) & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
  c=(char)((t>>4) & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
  c=(char)(t & 0x0F);
  if (c>9) c+=7;
  sw_uart_putc(c+'0');
}

/**------------------------------------------------------------
 * Decimlis kirats adott szm tizedesjegyre.
 *-------------------------------------------------------------
 * data - a kirand eljeles szm (int32_t)
 * ndigits - a kirand tizedesek szma (uint8_t)
 */
void sw_uart_outdec(int32_t data, uint8_t ndigits) {
  static char sign, s[12];
  int8_t i;
  i=0; sign=' ';
  if(data<0) { sign='-'; data = -data;}
    do {
      s[i]=data%10 + '0';
      data=data/10;
      i++;
      if(i==ndigits) {s[i]='.'; i++;}
    } while(data>0 || i<(ndigits+2));
    sw_uart_putc(sign);
    do {
      sw_uart_putc(s[--i]);
    } while(i);
}

int main( void ) {
//-- WDT s SMCLK inicializls ------  
  WDTCTL = WDTPW + WDTHOLD;            // Letiltjuk a watchdog idztt
  DCOCTL = CALDCO_1MHZ;                // DCO belltsa a gyrilag kalibrlt 
  BCSCTL1 = CALBC1_1MHZ;               // 1 MHz-es frekvencira
//-- Port1 konfigurls --------------  
  P1DIR |= TXD+CSEL+SCK;               // TXD, CSEL s SCK legyen digitlis kimenet
  P1DIR &= ~(SW2+SDI);                 // SW2 s SDI legyenek bemenetek  
  P1OUT |= TXD+SW2;                    // TXD=mark, SW2 felhzs
  P1REN |= SW2;                        // Bels felhzs engedlyezs
 
  while(1)  {

    P1OUT &= ~BIT4;                    // CSEL aktvls
    while(P1IN & SDI);                 // Vrakozs, ha az ADC foglalt
    data = 0;
    counter = 32;
    while(counter--)
    {
      data = data << 1;               // adatbitek lptetse balra
      P1OUT |= SCK;                   // rajel aktivls
      if(P1IN & SDI) data++;          // soron kvetkez adat bit beolvassa
      P1OUT &= ~SCK;                  // rajel alaphelyzetbe llts
    }
    P1OUT |= CSEL;                    // CSEL alaphelyzetbe llts
//-- nyers adat szeletelse
    d1 = data>>16;
    d2 = data & 0xFFFF;
    adc_sign = !(d1&0x2000);          // Eljel a 29 bit negltja
    adc_ext = (d1>>12)&1;             // Extrm rtk jelzse
    w = data & 0x1FFFFFFFUL;          // 28+1 bit a konverzi eredmnye
    if(adc_sign) {
      w |= 0xFFFFFFFFE0000000UL;
    }  
    w = w*2500000UL;                  // 2 500 000 uV
    u = (w>>28);                      // divide by 2^28 (ADC resolution)
    sw_uart_puts("\r\nadat = ");
    sw_uart_out4hex(d1);
    sw_uart_putc('_');
    sw_uart_out4hex(d2);
    sw_uart_puts("  fesz = ");
    sw_uart_outdec(u,6);
 //   sw_uart_puts(" V  Bci kplete =");
 //   sw_uart_outdec( (int32_t)(w * 2500000 / 0xFFFFFFF), 3);
    sw_uart_puts(" V");
    if(adc_ext) sw_uart_puts(" ***");
 //   __delay_cycles(1000000);
  }
}